@@ -167,6 +167,8 @@ sub computePlatformDefault($) {
     } elsif ($OSNAME eq "aix" || $OSNAME eq "freebsd" || $OSNAME eq "darwin" ||
              $OSNAME eq "amigaos") {
         $$defaultP = $OSNAME;
+    } elsif ($OSNAME eq "dec_osf") {
+        $$defaultP = "tru64";
     } else {
         print("Unrecognized OSNAME='$OSNAME'.  No default possible\n");
     }
@@ -293,10 +295,15 @@ sub getCompiler($) {
 # GNU compiler or undefined to express no preference.
 #-----------------------------------------------------------------------------
     my ($platform) = @_;
+    
+    my %gccCapablePlatform = ("SOLARIS" => 1,
+                              "TRU64"   => 1,
+                              "SCO"     => 1,
+                              "AIX"     => 1);
 
     my $compiler;
 
-    if ($platform eq "SOLARIS" || $platform eq "TRU64" || $platform eq "SCO") {
+    if ($gccCapablePlatform{$platform}) {
         print("GNU compiler or native operating system compiler (cc)?\n");
         print("\n");
 
@@ -328,7 +335,9 @@ sub getCompiler($) {
                   "set the CC environment variable or CC " .
                   "makefile variable or install 'gcc'\n");
         }
+        print("\n");
     }
+
     return($compiler);
 }
 
@@ -494,7 +503,11 @@ sub getPngLibrary($@) {
     if (defined($pnglib)) {
         my $default;
 
-        $default = "default";
+        if (-d("/usr/include/png")) {
+            $default = "/usr/include/libpng";
+        } else {
+            $default = "default";
+        }
         
         print("Where are the interface headers for it?\n");
         
@@ -529,7 +542,11 @@ sub getZLibrary($@) {
     if (defined($zlib)) {
         my $default;
 
-        $default = "default";
+        if (-d("/usr/include/zlib")) {
+            $default = "/usr/include/zlib";
+        } else {
+            $default = "default";
+        }
         
         print("Where are the interface headers for it?\n");
         
@@ -859,7 +876,7 @@ sub testCompileJpeglibH($$) {
     # Note: we tried using /dev/null for the output file and got complaints
     # from the Sun compiler that it has the wrong suffix.  2002.08.09.
     
-    my $compileCommand = "$cc -c -o$oFileName $cflags $cFileName";
+    my $compileCommand = "$cc -c -o $oFileName $cflags $cFileName";
     print ("Doing test compile: $compileCommand\n");
     my $rc = system($compileCommand);
     
@@ -891,7 +908,7 @@ sub testCompileJpegMarkerStruct($$) {
     # Note: we tried using /dev/null for the output file and got complaints
     # from the Sun compiler that it has the wrong suffix.  2002.08.09.
     
-    my $compileCommand = "$cc -c -o$oFileName $cflags $cFileName";
+    my $compileCommand = "$cc -c -o $oFileName $cflags $cFileName";
     print ("Doing test compile: $compileCommand\n");
     my $rc = system($compileCommand);
     
@@ -1054,6 +1071,8 @@ displayIntroduction();
 
 my ($platform, $subplatform) = getPlatform();
 
+print("\n");
+
 if ($platform eq "NONE") {
     print("You will have to construct Makefile.config manually.  To do \n");
     print("this, copy Makefile.config.in as Makefile.config, and then \n");
@@ -1144,9 +1163,6 @@ print("\n");
 	    }
             if ($platform eq "AIX") {
                 $netpbmlibsuffix = "a";
-                print("I don't know how to make a shared library on AIX.  " .
-                      "If you do, please clue in the Netpbm maintainer.\n");
-                exit(1)
             } elsif ($platform eq "HP-UX") {
                 $netpbmlibsuffix = "sl";
             } else {
@@ -1169,13 +1185,16 @@ print("\n");
 
 my $staticlib_too;
 
-if ($netpbmlibtype eq "unixshared" or 
-    $netpbmlibtype eq "irixshared" or 
-    $netpbmlibtype eq "dll") {
+# Note that we can't do both a static and shared library for AIX, because
+# they both have the same name: libnetpbm.a.
+
+if (($netpbmlibtype eq "unixshared" or 
+     $netpbmlibtype eq "irixshared" or 
+     $netpbmlibtype eq "dll") and $netpbmlibsuffix ne "a") {
     print("Do you want to build static libraries too (for linking to \n");
     print("programs not in the Netpbm package?\n");
     print("\n");
-
+    
     my $default = "y";
 
     my $response = prompt("(y)es or (n)o", $default);
@@ -1330,7 +1349,8 @@ if (-f("GNUmakefile")) {
 #*****************************************************************************
 
 sub gnuCflags() {
-    return("CFLAGS = -pedantic " . gnuOptimizeOpt() . " " .
+    return("CFLAGS = " . gnuOptimizeOpt() . " -ffast-math " .
+           " -pedantic " . 
            "-Wall -Wno-uninitialized -Wmissing-declarations -Wimplicit " .
            "-Wwrite-strings\n");
 }
@@ -1402,25 +1422,32 @@ if ($platform eq "GNU") {
     if ($compiler eq "cc") {
         push(@Makefile_config, "CFLAGS = -O\n");
         push(@Makefile_config, "CFLAGS_SHLIB = -Kpic\n");
+        push(@Makefile_config, "RPATHOPTNAME = -R\n");
     } else {
         push(@Makefile_config, "CC = gcc\n");
         push(@Makefile_config, gnuCflags());
+        push(@Makefile_config, "RPATHOPTNAME = -rpath\n");
     }
     push(@Makefile_config, "NETWORKLD = -lsocket -lnsl\n");
 } elsif ($platform eq "HP-UX") {
     push(@Makefile_config, 'CFLAGS = -O3 -fPIC $(CADD)', "\n");
     push(@Makefile_config, 'LDFLAGS = -Wl,+b,/usr/pubsw/lib', "\n");
 } elsif ($platform eq "AIX") {
-    # These options seem to be for gcc; I don't know what to use for any
-    # other compiler; I hope someone tells us so we can do like we did 
-    # for SOLARIS above
-    push(@Makefile_config, 'CFLAGS = -O3 -D_ALL_SOURCE $(CADD)', "\n");
     push(@Makefile_config, 'LDFLAGS = -L /usr/pubsw/lib', "\n");
-    push(@Makefile_config, 'LDSHLIB = -shared', "\n");
+    if ($compiler eq "cc") {
+        # Yes, the -L option implies the runtime as well as linktime library
+        # search path.  There's no way to specify runtime path independently.
+        push(@Makefile_config, "RPATHOPTNAME = -L\n");
+        push(@Makefile_config, "LDSHLIB = -qmkshrobj\n");
+    } else {
+        push(@Makefile_config, "CC = gcc\n");
+        push(@Makefile_config, gnuCflags());
+        push(@Makefile_config, "LDSHLIB = -shared\n");
+    }
 } elsif ($platform eq "TRU64") {
 #    push(@Makefile_config, "INSTALL = installbsd\n");
     if ($compiler eq "cc") {
-        push(@Makefile_config, 'CFLAGS = -O2 -std1 -DLONG_32 $(CADD)', "\n");
+        push(@Makefile_config, 'CFLAGS = -O2 -std1 $(CADD)', "\n");
         push(@Makefile_config, "LDFLAGS = -call_shared -oldstyle_liblookup " .
              "-L/usr/local/lib\n");
         push(@Makefile_config, "LDSHLIB = -shared -expect_unresolved \"*\"\n");
@@ -1428,9 +1455,15 @@ if ($platform eq "GNU") {
         # We've never tested this.  This is just here to give a user a 
         # headstart on submitting to us the necessary information.  2002.07.04.
         push(@Makefile_config, "CC = gcc\n");
-        push(@Makefile_config, 'CFLAGS = -O3 -DLONG_32 $(CADD)', "\n");
+        push(@Makefile_config, 'CFLAGS = -O3 $(CADD)', "\n");
         push(@Makefile_config, "LDSHLIB = -shared\n");
     }
+    # Between May 2000 and July 2003, we had -DLONG_32 in these options.
+    # We took it out because it generated bad code for a TRU64 user in
+    # July 2003 whose system has 64 bit long and 32 bit int.  It affects
+    # only Ppmtompeg and it isn't clear that using long instead of int is
+    # ever right anyway.
+
     push(@Makefile_config, "OMIT_NETWORK = y\n");
     push(@Makefile_config, "LINKER_CAN_DO_EXPLICIT_LIBRARY=Y\n");
 } elsif ($platform eq "IRIX") {
@@ -1505,13 +1538,12 @@ if ($platform eq "GNU") {
 }
 
 if (needLocal($platform)) {
-    push(@Makefile_config, "CFLAGS += -I/usr/local/include");
-    push(@Makefile_config, "LDFLAGS += -L/usr/local/lib");
+    push(@Makefile_config, "CFLAGS += -I/usr/local/include\n");
+    push(@Makefile_config, "LDFLAGS += -L/usr/local/lib\n");
 }
 
 if (defined($linker) && $linker eq "SUNVIACOMPILER") {
-    push(@Makefile_config, "LINKERISCOMPILER = Y");
-    push(@Makefile_config, "RPATHOPTNAME = -R");
+    push(@Makefile_config, "LINKERISCOMPILER = Y\n");
 }
 
 my $flex_result = `flex --version`;
