@@ -19,6 +19,8 @@
 
 *****************************************************************************/
 #include <string.h>
+#include <limits.h>
+#include <assert.h>
 
 #include "pnm.h"
 #include "shhopt.h"
@@ -33,6 +35,63 @@ static xelval const bmpMaxval = 255;
        truecolor raster or in a colormap
     */
 
+enum rowOrder {BOTTOMUP, TOPDOWN};
+
+struct bitPosition {
+    /* mask and shift count to describe a set of bits in a binary value.
+
+       Example: if 16 bits are laid out as XRRRRRGGGGGBBBBB then the shift
+       count for the R component is 10 and the mask is 0000000000011111.
+    */
+    unsigned int shift;
+        /* How many bits right you have to shift the value to get the subject
+           bits in the least significant bit positions.
+        */
+    unsigned int mask;
+        /* Has one bits in positions where the subject bits are after
+           shifting.
+        */
+};
+
+struct pixelformat {
+    /* The format of a pixel representation from the raster.  i.e. which 
+       bits apply to red, green, blue, and transparency 
+    */
+    struct bitPosition red;
+    struct bitPosition blu;
+    struct bitPosition grn;
+    struct bitPosition trn;
+    
+    bool conventionalBgr;
+        /* This means that the above bit positions are just the conventional
+           BGR format -- one byte Blue, one byte Green, one byte Red,
+           no alpha.  Though it's totally redundant with the members above,
+           this member speeds up computation:  We've never actually seen
+           a BMP file that doesn't use conventional BGR, and it doesn't
+           require any masking or shifting at all to interpret.
+        */
+};
+
+struct bmpInfoHeader {
+    enum rowOrder rowOrder;
+    int cols;
+    int rows;
+    unsigned int cBitCount;
+        /* Number of bits in the BMP file that each pixel occupies. */
+    enum bmpClass class;
+    bool bitFields;
+        /* The raster values are arranged in arbitrary bit fields as
+           described by the "mask" values in the header, rather than
+           fixed formats.
+        */
+    int cmapsize;
+        /* Size in bytes of the colormap (palette) in the BMP file */
+    unsigned short cPlanes;
+    struct pixelformat pixelformat;
+};
+
+
+
 struct cmdline_info {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
@@ -84,7 +143,7 @@ parse_command_line(int argc, char ** argv,
 
 
 
-static char     er_read[] = "%s: read error";
+static const char er_read[] = "%s: read error";
 
 static int
 GetByte(FILE * const fp) {
@@ -125,6 +184,25 @@ GetLong(FILE * const fp) {
 
 
 
+typedef struct {
+    long dummy[12];
+} cieXyzTriple;
+
+static cieXyzTriple
+GetCieXyzTriple(FILE * const fp) {
+
+    cieXyzTriple v;
+    unsigned int i;
+
+    for (i = 0; i < 12; ++i) 
+        if (pm_readlittlelong(fp, &v.dummy[i]) == -1)
+            pm_error(er_read, ifname);
+
+    return v;
+}
+
+
+
 static void
 readOffBytes(FILE * const fp, unsigned int const nbytes) {
 /*----------------------------------------------------------------------------
@@ -168,135 +246,288 @@ BMPreadfileheader(FILE * const ifP,
 }
 
 
-struct BMPheader {
-    int cols;
-    int rows;
-    unsigned int cBitCount;
-    enum bmpClass class;
-    int cmapsize;
-    unsigned short cPlanes;
-};
+
+static void
+readOs2InfoHeader(FILE *                 const ifP,
+                  struct bmpInfoHeader * const headerP) {
+
+    headerP->class = C_OS2;
+
+    headerP->cols = GetShort(ifP);
+    headerP->rows = GetShort(ifP);
+    headerP->rowOrder = BOTTOMUP;
+    headerP->cPlanes = GetShort(ifP);
+    headerP->cBitCount = GetShort(ifP);
+    /* I actually don't know if the OS/2 BMP format allows
+       cBitCount > 8 or if it does, what it means, but ppmtobmp
+       creates such BMPs, more or less as a byproduct of creating
+       the same for Windows BMP, so we interpret cBitCount > 8 the
+       same as for Windows.
+    */
+    if (headerP->cBitCount <= 8)
+        headerP->cmapsize = 1 << headerP->cBitCount;
+    else if (headerP->cBitCount == 24)
+        headerP->cmapsize = 0;
+    /* There is a 16 bit truecolor format, but we don't know how the
+       bits are divided among red, green, and blue, so we can't handle it.
+    */
+    else
+        pm_error("Unrecognized bits per pixel in OS/2 BMP file header: %d",
+                 headerP->cBitCount);
+
+    pm_message("OS/2 BMP, %dx%dx%d",
+               headerP->cols, headerP->rows, headerP->cBitCount);
+}
+
 
 
 static void
-BMPreadinfoheader(FILE *             const ifP, 
-                  unsigned int *     const bytesReadP,
-                  struct BMPheader * const headerP) {
+readWindowsBasic40ByteInfoHeader(FILE *                 const ifP,
+                                 struct bmpInfoHeader * const headerP) {
 
-    unsigned long   cbFix;
-    unsigned short  cPlanes;
+    int colorsimportant;   /* ColorsImportant value from header */
+    int colorsused;        /* ColorsUsed value from header */
 
-    unsigned long   cx;
-    unsigned long   cy;
-    unsigned short  cBitCount;
-    unsigned long   compression;
-    int             class;
+    headerP->class = C_WIN;
 
-    cbFix = GetLong(ifP);
-
-    switch (cbFix) {
-    case 12:
-        class = C_OS2;
-
-        cx = GetShort(ifP);
-        cy = GetShort(ifP);
-        cPlanes = GetShort(ifP);
-        cBitCount = GetShort(ifP);
-        /* I actually don't know if the OS/2 BMP format allows
-           cBitCount > 8 or if it does, what it means, but ppmtobmp
-           creates such BMPs, more or less as a byproduct of creating
-           the same for Windows BMP, so we interpret cBitCount > 8 the
-           same as for Windows.
-        */
-        if (cBitCount <= 8)
-            headerP->cmapsize = 1 << cBitCount;
-        else if (cBitCount == 24)
-            headerP->cmapsize = 0;
-        /* There is a 16 bit truecolor format, but we don't know how the
-           bits are divided among red, green, and blue, so we can't handle it.
-        */
-        else
-            pm_error("Unrecognized bits per pixel in OS/2 BMP file header: %d",
-                     cBitCount);
-
-        break;
-    case 40: {
-        int colorsimportant;   /* ColorsImportant value from header */
-        int colorsused;        /* ColorsUsed value from header */
-
-        class = C_WIN;
-
-        cx = GetLong(ifP);
-        cy = GetLong(ifP);
-        cPlanes = GetShort(ifP);
-        cBitCount = GetShort(ifP);
+    headerP->cols = GetLong(ifP);
+    {
+        long const cy = GetLong(ifP);
+        if (cy < 0) {
+            headerP->rowOrder = TOPDOWN;
+            headerP->rows = - cy;
+        } else {
+            headerP->rowOrder = BOTTOMUP;
+            headerP->rows = cy;
+        }
+    }
+    headerP->cPlanes = GetShort(ifP);
+    headerP->cBitCount = GetShort(ifP);
  
-        compression = GetLong(ifP);
-        if (compression != 0) 
+    {
+        unsigned long const compression = GetLong(ifP);
+
+        headerP->bitFields = (compression == COMP_BITFIELDS);
+            
+        if (compression != COMP_RGB && compression != COMP_BITFIELDS) 
             pm_error("Input is compressed.  This program doesn't handle "
                      "compressed images.  Compression type code = %ld",
                      compression);
-
-        /* And read the rest of the junk in the 40 byte header */
-        GetLong(ifP);   /* ImageSize */
-        GetLong(ifP);   /* XpixelsPerM */
-        GetLong(ifP);   /* YpixelsPerM */
-        colorsused = GetLong(ifP);   /* ColorsUsed */
-        /* See comments in bmp.h for info about the definition of the following
-           word and its relationship to the color map size (*pcmapsize).
-           */
-        colorsimportant = GetLong(ifP);  /* ColorsImportant */
-
-        if (cBitCount <= 8) {
-            if (colorsused != 0) {
-                if (colorsused > 1 << cBitCount)
-                    pm_error("Invalid BMP header.  Says %u bits per pixel, "
-                             "but %d colors used", cBitCount, colorsused);
-                else
-                    headerP->cmapsize = colorsused;
-            } else 
-                headerP->cmapsize = 1 << cBitCount;
-        } else if (cBitCount == 24 || cBitCount == 16)
-            headerP->cmapsize = 0;
-        else
-            pm_error("Unrecognized bits per pixel in "
-                     "Windows BMP file header: %d",
-                     cBitCount);
     }
+    /* And read the rest of the junk in the 40 byte header */
+    GetLong(ifP);   /* ImageSize */
+    GetLong(ifP);   /* XpixelsPerMeter */
+    GetLong(ifP);   /* YpixelsPerMeter */
+    colorsused = GetLong(ifP);   /* ColorsUsed */
+    /* See comments in bmp.h for info about the definition of the following
+       word and its relationship to the color map size (*pcmapsize).
+    */
+    colorsimportant = GetLong(ifP);  /* ColorsImportant */
+
+    if (headerP->cBitCount <= 8) {
+        if (colorsused != 0) {
+            if (colorsused > 1 << headerP->cBitCount)
+                pm_error("Invalid BMP header.  Says %u bits per pixel, "
+                         "but %d colors used", 
+                         headerP->cBitCount, colorsused);
+            else
+                headerP->cmapsize = colorsused;
+        } else 
+            headerP->cmapsize = 1 << headerP->cBitCount;
+    } else if (headerP->cBitCount == 24 || 
+               headerP->cBitCount == 16 || 
+               headerP->cBitCount == 32)
+        headerP->cmapsize = 0;
+    else
+        pm_error("Unrecognized bits per pixel in Windows BMP file header: %d",
+                 headerP->cBitCount);
+}
+
+
+
+static unsigned int
+lsbZeroCount(unsigned int const mask) {
+/*----------------------------------------------------------------------------
+   Return the number of consecutive zeroes in the mask 'mask', starting with
+   the least significant bit and going up.  E.g. for 0x1F, it would be 5.
+-----------------------------------------------------------------------------*/
+    unsigned int i;
+
+    i = 0;
+    
+    while (((mask >> i) & 0x1) == 0 && i < sizeof(mask)*8)
+        ++i;
+
+    return i;
+}
+
+
+
+static struct bitPosition
+bitPositionFromMask(long const bmpMask) {
+    struct bitPosition retval;
+
+    retval.shift = lsbZeroCount(bmpMask);
+    retval.mask  = bmpMask >> retval.shift;
+
+    return retval;
+}
+
+
+
+static void
+computeConventionalBgr(struct pixelformat * const fP,
+                       unsigned int         const bitCount) {
+
+    switch (bitCount) {
+    case 24:
+        fP->conventionalBgr =
+            fP->red.shift ==  0 && fP->red.mask == 0xFF &&
+            fP->grn.shift ==  8 && fP->grn.mask == 0xFF &&
+            fP->blu.shift == 16 && fP->blu.mask == 0xFF &&
+            fP->trn.mask == 0
+            ;
+        break;
+    case 32:
+        fP->conventionalBgr =
+            fP->red.shift ==  8  && fP->red.mask == 0xFF &&
+            fP->grn.shift == 16  && fP->grn.mask == 0xFF &&
+            fP->blu.shift == 24  && fP->blu.mask == 0xFF &&
+            fP->trn.mask == 0
+            ;
         break;
     default:
-        pm_error("%s: unknown cbFix: %ld", ifname, cbFix);
+        fP->conventionalBgr = FALSE;
+    }
+}
+
+
+
+static struct pixelformat
+defaultPixelformat(unsigned int const bitCount) {
+
+    struct pixelformat retval;
+
+    switch (bitCount) {
+    case 16:
+        retval.conventionalBgr = FALSE;
+        retval.red.shift = 10;
+        retval.grn.shift = 5;
+        retval.blu.shift = 0;
+        retval.trn.shift = 0;
+        retval.red.mask = 0x1f;  /* 5 bits */
+        retval.grn.mask = 0x1f;  /* 5 bits */
+        retval.blu.mask = 0x1f;  /* 5 bits */
+        retval.trn.mask = 0;
+        break;
+    case 24:
+    case 32:
+        retval.conventionalBgr = TRUE;
+        retval.red.shift = 16;
+        retval.grn.shift = 8;
+        retval.blu.shift = 0;
+        retval.trn.shift = 0;
+        retval.red.mask = 0xff;  /* 8 bits */
+        retval.grn.mask = 0xff;  /* 8 bits */
+        retval.blu.mask = 0xff;  /* 8 bits */
+        retval.trn.mask = 0;
+        break;
+    default:
+        /* colormapped - masks are undefined */
         break;
     }
 
-    if (cPlanes != 1) {
-        pm_error("%s: don't know how to handle cPlanes = %d"
-             ,ifname
-             ,cPlanes);
+    return retval;
+}
+
+
+
+static void
+readV4InfoHeaderExtension(FILE *                 const ifP, 
+                          struct bmpInfoHeader * const headerP) {
+
+    if (headerP->bitFields) {
+        headerP->pixelformat.red = bitPositionFromMask(GetLong(ifP));
+        headerP->pixelformat.grn = bitPositionFromMask(GetLong(ifP));
+        headerP->pixelformat.blu = bitPositionFromMask(GetLong(ifP));
+        headerP->pixelformat.trn = bitPositionFromMask(GetLong(ifP));
+
+        computeConventionalBgr(&headerP->pixelformat, headerP->cBitCount);
+    } else
+        headerP->pixelformat = defaultPixelformat(headerP->cBitCount);
+
+    GetLong(ifP);  /* Color space */
+    GetCieXyzTriple(ifP);  /* Endpoints */
+    GetLong(ifP);  /* GammaRed */
+    GetLong(ifP);  /* GammaGreen */
+    GetLong(ifP);  /* GammaBlue */
+} 
+
+
+
+static void
+defaultV4InfoHeaderExtension(struct bmpInfoHeader * const headerP) {
+
+    headerP->pixelformat = defaultPixelformat(headerP->cBitCount);
+
+}
+
+
+
+static void
+readWindowsInfoHeader(FILE *                 const ifP, 
+                      unsigned int           const cInfoHeaderSize,
+                      struct bmpInfoHeader * const headerP) {
+
+    /* There are 3 major formats of Windows
+       BMP, identified by the 3 info header lengths.  The original
+       one is 40 bytes.  The "V4 header" is 108 bytes and was
+       new with Windows 95 and NT 4.0.  The "V5 header" is 124 bytes
+       and was new with Windows 98 and Windows 2000.
+    */
+    readWindowsBasic40ByteInfoHeader(ifP, headerP);
+
+    if (cInfoHeaderSize >= 108) 
+        readV4InfoHeaderExtension(ifP, headerP);
+    else 
+        defaultV4InfoHeaderExtension(headerP);
+
+    if (cInfoHeaderSize >= 124) {
+        /* Read off the V5 info header extension. */
+        GetLong(ifP);  /* Intent */
+        GetLong(ifP);  /* ProfileData */
+        GetLong(ifP);  /* ProfileSize */
+        GetLong(ifP);  /* Reserved */
     }
 
-    switch (class) {
-    case C_WIN:
-        pm_message("Windows BMP, %ldx%ldx%d"
-               ,cx
-               ,cy
-               ,cBitCount);
+    pm_message("Windows BMP, %dx%dx%d",
+               headerP->cols, headerP->rows, headerP->cBitCount);
+}
+
+
+
+static void
+BMPreadinfoheader(FILE *                 const ifP, 
+                  unsigned int *         const bytesReadP,
+                  struct bmpInfoHeader * const headerP) {
+
+    unsigned int const cInfoHeaderSize = GetLong(ifP);
+
+    switch (cInfoHeaderSize) {
+    case 12:
+        readOs2InfoHeader(ifP, headerP);
         break;
-    case C_OS2:
-        pm_message("OS/2 BMP, %ldx%ldx%d"
-               ,cx
-               ,cy
-               ,cBitCount);
+    case 40: 
+    case 108:
+    case 124:
+        readWindowsInfoHeader(ifP, cInfoHeaderSize, headerP);
+        break;
+    default:
+        pm_error("%s: unknown Info Header size: %u bytes", 
+                 ifname, cInfoHeaderSize);
         break;
     }
-
-    headerP->cols = cx;
-    headerP->rows = cy;
-    headerP->cBitCount = cBitCount;
-    headerP->class = class;
-    headerP->cPlanes = cPlanes;
-
-    *bytesReadP = cbFix;
+    *bytesReadP = cInfoHeaderSize;
 }
 
 
@@ -344,31 +575,49 @@ BMPreadcolormap(FILE *         const ifP,
 
 
 static void
-convertRow16(unsigned char const bmprow[],
-             xel                 xelrow[],
-             int           const cols) {
-    /* It's truecolor.  We haven't seen a spec for this format, but we
-       have seen one example of it, produced by National Instruments
-       Lookout, on 2003.06.09, and the encoding of the 16 bit raster
-       element was apparently as follows: It's a signed little-endian
-       integer, which is never negative, and the 15 bit binary
-       representation of that integer can be decomposed into 5 bits
-       each of red, green, and blue going from msb to lsb.  
-    */
+extractBitFields(unsigned int       const rasterval,
+                 struct pixelformat const pixelformat,
+                 pixval             const maxval,
+                 pixval *           const rP,
+                 pixval *           const gP,
+                 pixval *           const bP,
+                 pixval *           const aP) {
+
+    unsigned int const rbits = 
+        (rasterval >> pixelformat.red.shift) & pixelformat.red.mask;
+    unsigned int const gbits = 
+        (rasterval >> pixelformat.grn.shift) & pixelformat.grn.mask;
+    unsigned int const bbits = 
+        (rasterval >> pixelformat.blu.shift) & pixelformat.blu.mask;
+    unsigned int const abits = 
+        (rasterval >> pixelformat.trn.shift) & pixelformat.trn.mask;
+    
+    *rP = (unsigned int) rbits * maxval / pixelformat.red.mask;
+    *gP = (unsigned int) gbits * maxval / pixelformat.blu.mask;
+    *bP = (unsigned int) bbits * maxval / pixelformat.grn.mask;
+    *aP = (unsigned int) abits * maxval / pixelformat.trn.mask;
+}        
+
+
+
+static void
+convertRow16(unsigned char      const bmprow[],
+             xel                      xelrow[],
+             int                const cols,
+             struct pixelformat const pixelformat) {
+    /* It's truecolor.  */
+
     unsigned int col;
     unsigned int cursor;
     cursor = 0;
     for (col=0; col < cols; ++col) {
         unsigned short const rasterval = (unsigned short)
             bmprow[cursor+1] << 8 | bmprow[cursor+0];
-        unsigned int const rbits = (rasterval >> 10) & 0x1F;
-        unsigned int const gbits = (rasterval >> 5)  & 0x1F;
-        unsigned int const bbits = (rasterval >> 0)  & 0x1F;
 
-        pixval const r = (unsigned int) rbits * 255 / 31;
-        pixval const g = (unsigned int) gbits * 255 / 31;
-        pixval const b = (unsigned int) bbits * 255 / 31;
-        
+        pixval r, g, b, a;
+
+        extractBitFields(rasterval, pixelformat, 255, &r, &g, &b, &a);
+
         PNM_ASSIGN(xelrow[col], r, g, b);
         
         cursor += 2;
@@ -378,20 +627,39 @@ convertRow16(unsigned char const bmprow[],
 
 
 static void
-convertRow24(unsigned char const bmprow[],
-             xel                 xelrow[],
-             int           const cols) {
+convertRow24(unsigned char      const bmprow[],
+             xel                      xelrow[],
+             int                const cols,
+             struct pixelformat const pixelformat) {
     
     /* It's truecolor */
     /* There is a document that gives a much different format for
-       24 bit BMPs.  But this seems to be the de facto standard.
+       24 bit BMPs.  But this seems to be the de facto standard, and is,
+       with a little ambiguity and contradiction resolved, defined in the
+       Microsoft BMP spec.
     */
+
     unsigned int col;
     unsigned int cursor;
+    
     cursor = 0;
     for (col = 0; col < cols; ++col) {
-        PNM_ASSIGN(xelrow[col], 
-                   bmprow[cursor+2], bmprow[cursor+1], bmprow[cursor+0]);
+        pixval r, g, b, a;
+
+        if (pixelformat.conventionalBgr) {
+            r = bmprow[cursor+2];
+            g = bmprow[cursor+1];
+            b = bmprow[cursor+0];
+            a = 0;
+        } else {
+            unsigned int const rasterval = 
+                (bmprow[cursor+0] << 16) +
+                (bmprow[cursor+1] << 8) +
+                (bmprow[cursor+2] << 0);
+            
+            extractBitFields(rasterval, pixelformat, 255, &r, &g, &b, &a);
+        }
+        PNM_ASSIGN(xelrow[col], r, g, b);
         cursor += 3;
     }
 } 
@@ -399,24 +667,67 @@ convertRow24(unsigned char const bmprow[],
 
 
 static void
-convertRow(unsigned char  const bmprow[], 
-           xel                  xelrow[],
-           int            const cols, 
-           unsigned int   const cBitCount, 
-           xel            const colormap[]
+convertRow32(unsigned char      const bmprow[],
+             xel                      xelrow[],
+             int                const cols,
+             struct pixelformat const pixelformat) {
+    
+    /* It's truecolor */
+
+    unsigned int col;
+    unsigned int cursor;
+    cursor = 0;
+    for (col = 0; col < cols; ++col) {
+        pixval r, g, b, a;
+
+        if (pixelformat.conventionalBgr) {
+            /* bmprow[cursor+3] is just padding */
+            r = bmprow[cursor+2];
+            g = bmprow[cursor+1];
+            b = bmprow[cursor+0];
+            a = 0;
+        } else {
+            unsigned int const rasterval = 
+                (bmprow[cursor+0] << 24) +
+                (bmprow[cursor+1] << 16) +
+                (bmprow[cursor+2] << 8) +
+                (bmprow[cursor+3] << 0);
+            
+            extractBitFields(rasterval, pixelformat, 255, &r, &g, &b, &a);
+        }
+
+        PNM_ASSIGN(xelrow[col], 
+                   bmprow[cursor+2], bmprow[cursor+1], bmprow[cursor+0]);
+        cursor += 4;
+    }
+} 
+
+
+
+static void
+convertRow(unsigned char      const bmprow[], 
+           xel                      xelrow[],
+           int                const cols, 
+           unsigned int       const cBitCount, 
+           struct pixelformat const pixelformat,
+           xel                const colormap[]
            ) {
 /*----------------------------------------------------------------------------
    Convert a row in raw BMP raster format bmprow[] to a row of xels xelrow[].
 
+   Use maxval 255 for the output xels.
+
    The BMP image has 'cBitCount' bits per pixel.
 
    If the image is colormapped, colormap[] is the colormap
    (colormap[i] is the color with color index i).
 -----------------------------------------------------------------------------*/
     if (cBitCount == 24) 
-        convertRow24(bmprow, xelrow, cols);
+        convertRow24(bmprow, xelrow, cols, pixelformat);
     else if (cBitCount == 16) 
-        convertRow16(bmprow, xelrow, cols);
+        convertRow16(bmprow, xelrow, cols, pixelformat);
+    else if (cBitCount == 32) 
+        convertRow32(bmprow, xelrow, cols, pixelformat);
     else if (cBitCount == 8) {            
         /* It's a whole byte colormap index */
         unsigned int col;
@@ -442,7 +753,7 @@ convertRow(unsigned char  const bmprow[],
 
 
 static unsigned char **
-allocBMPraster(int const rows, unsigned int const bytesPerRow) {
+allocBMPraster(unsigned int const rows, unsigned int const bytesPerRow) {
 
     unsigned int const storageSize = 
         rows * sizeof(unsigned char *) + rows * bytesPerRow;        
@@ -454,6 +765,9 @@ allocBMPraster(int const rows, unsigned int const bytesPerRow) {
        followed by the rows of bytes, in a single allocated chunk of storage.
     */
 
+    if (UINT_MAX / (bytesPerRow + sizeof(unsigned char *)) < rows)
+        pm_error("raster is ridiculously large.");
+
     BMPraster = (unsigned char **) malloc(storageSize);
 
     if (BMPraster == NULL)
@@ -469,89 +783,87 @@ allocBMPraster(int const rows, unsigned int const bytesPerRow) {
 }
 
 
+
 static void
-BMPreadraster(FILE * const ifP, 
-              int const cols, int const rows, 
-              unsigned int const cBitCount, enum bmpClass const class, 
-              xel colormap[], unsigned char *** const BMPrasterP, 
-              unsigned int * const bytesReadP) {
+readrow(FILE *           const ifP,
+        unsigned int     const row,
+        unsigned int     const bytesPerRow,
+        unsigned char ** const BMPraster,
+        unsigned int *   const bytesReadP) {
+
+    size_t totalBytesRead;
+    
+    totalBytesRead = 0;
+
+    while (totalBytesRead < bytesPerRow) {
+        size_t bytesRead;
+
+        assert(bytesPerRow > 0);
+    
+        bytesRead = fread(BMPraster[row], 1, bytesPerRow, ifP);
+        if (bytesRead == 0) {
+            if (feof(ifP))
+                pm_error("End of file reading row %u of BMP raster.", row);
+            else 
+                pm_error("Error reading BMP raster.  Errno=%d (%s)",
+                         errno, strerror(errno));
+        }
+        totalBytesRead += bytesRead;
+    }
+    *bytesReadP += totalBytesRead;
+}
+
+
+
+static void
+BMPreadraster(FILE *            const ifP, 
+              unsigned int      const cols, 
+              unsigned int      const rows, 
+              enum rowOrder     const rowOrder,
+              unsigned int      const cBitCount, 
+              enum bmpClass     const class, 
+              xel                     colormap[], 
+              unsigned char *** const BMPrasterP, 
+              unsigned int *    const bytesReadP) {
 
     unsigned int const bytesPerRow = ((cols * cBitCount +31)/32)*4;
         /* A BMP raster row is a multiple of 4 bytes, padded on the right
            with don't cares.
         */
-    int row;
     unsigned char ** BMPraster;
 
     BMPraster = allocBMPraster(rows, bytesPerRow);
-    /*
-     * The picture is stored bottom line first, top line last
-     */
 
     *bytesReadP = 0;
 
-    for (row = rows - 1; row >= 0; --row) {
-        unsigned int bytesRead;
-        
-        bytesRead = 0;
+    switch (rowOrder) {
+    case BOTTOMUP: {
+        /* This is by far the most common case - the bottom line is first
+           in the file, the top line last.
+        */
+        int row;
 
-        while (bytesRead < bytesPerRow) {
-            int rc;
-            rc = fread(BMPraster[row], 1, bytesPerRow, ifP);
-            if (rc < 0) {
-                if (feof(ifP))
-                    pm_error("End of file reading row %u of BMP raster.", row);
-                else 
-                    pm_error("Error reading BMP raster.  Errno=%d (%s)",
-                             errno, strerror(errno));
-            }
-            bytesRead += rc;
-        }
-        *bytesReadP += bytesRead;
+        for (row = rows - 1; row >= 0; --row) 
+            readrow(ifP, row, bytesPerRow, BMPraster, bytesReadP);
+    
+    } break;
+    case TOPDOWN: {
+        /* We have never actually seen this, except in a Microsoft spec */
+
+        unsigned int row;
+        
+        for (row = 0; row < rows; ++row)
+            readrow(ifP, row, bytesPerRow, BMPraster, bytesReadP);
+    } break;
     }
+
     *BMPrasterP = BMPraster;
 }
 
 
 
 static void
-writeRaster(unsigned char ** const BMPraster,
-            int              const cols, 
-            int              const rows, 
-            int              const format,
-            unsigned int     const cBitCount, 
-            enum bmpClass    const class, 
-            xel                    colormap[]) {
-/*----------------------------------------------------------------------------
-  Write the PNM raster to Standard Output, corresponding to the raw BMP
-  raster BMPraster.  Write the raster assuming the PNM image has 
-  dimensions 'cols' by 'rows' and format 'format'.
-
-  The BMP image is of class 'class' and  has 'cBitCount' bits per pixel.
-
-  If the image is colormapped, colormap[] is the colormap
-  (colormap[i] is the color with color index i).
------------------------------------------------------------------------------*/
-    xel * xelrow;
-    int row;
-
-    xelrow = pnm_allocrow(cols);
-
-    /*
-     * The picture is stored bottom line first, top line last
-     */
-
-    for (row = 0; row < rows; ++row) {
-        convertRow(BMPraster[row], xelrow, cols, cBitCount, colormap);
-        pnm_writepnmrow(stdout, xelrow, cols, bmpMaxval, format, FALSE);
-    }
-    pnm_freerow(xelrow);
-}
-
-
-
-static void
-reportHeader(struct BMPheader const header, unsigned int const offBits) {
+reportHeader(struct bmpInfoHeader const header, unsigned int const offBits) {
     
     pm_message("BMP image header says:");
     pm_message("  Class of BMP: %s", 
@@ -561,6 +873,8 @@ reportHeader(struct BMPheader const header, unsigned int const offBits) {
     pm_message("  Width: %d pixels", header.cols);
     pm_message("  Height: %d pixels", header.rows);
     pm_message("  Depth: %d planes", header.cPlanes);
+    pm_message("  Row order: %s", 
+               header.rowOrder == BOTTOMUP ? "bottom up" : "top down");
     pm_message("  Byte offset of raster within file: %u", offBits);
     pm_message("  Bits per pixel in raster: %u", header.cBitCount);
     pm_message("  Colors in color map: %d", header.cmapsize);
@@ -601,16 +915,16 @@ analyzeColors(xel colormap[], unsigned int const cmapsize, xelval const maxval,
 
 
 static void
-readBmp(FILE *            const ifP, 
-        unsigned char *** const BMPrasterP, 
-        int *             const colsP, 
-        int *             const rowsP,
-        bool *            const grayPresentP, 
-        bool *            const colorPresentP,
-        unsigned int *    const cBitCountP, 
-        enum bmpClass *   const classP, 
-        xel **            const colormapP,
-        bool              const verbose) {
+readBmp(FILE *               const ifP, 
+        unsigned char ***    const BMPrasterP, 
+        int *                const colsP, 
+        int *                const rowsP,
+        bool *               const grayPresentP, 
+        bool *               const colorPresentP,
+        unsigned int *       const cBitCountP, 
+        struct pixelformat * const pixelformatP,
+        xel **               const colormapP,
+        bool                 const verbose) {
 
     xel * colormap;  /* malloc'ed */
     unsigned int pos;
@@ -618,7 +932,7 @@ readBmp(FILE *            const ifP,
     /* The following are all information from the BMP headers */
     unsigned int offBits;
         /* Byte offset into file of raster */
-    struct BMPheader BMPheader;
+    struct bmpInfoHeader BMPheader;
 
     pos = 0;
     { 
@@ -671,7 +985,7 @@ readBmp(FILE *            const ifP,
 
     {
         unsigned int bytesRead;
-        BMPreadraster(ifP, BMPheader.cols, BMPheader.rows, 
+        BMPreadraster(ifP, BMPheader.cols, BMPheader.rows, BMPheader.rowOrder,
                       BMPheader.cBitCount, BMPheader.class, colormap, 
                       BMPrasterP, &bytesRead);
         pos += bytesRead;
@@ -687,12 +1001,46 @@ readBmp(FILE *            const ifP,
     *colsP = BMPheader.cols;
     *rowsP = BMPheader.rows;
     *cBitCountP = BMPheader.cBitCount;
-    *classP = BMPheader.class;
+    *pixelformatP = BMPheader.pixelformat;
     *colormapP = colormap;
 }
 
 
 
+static void
+writeRaster(unsigned char **   const BMPraster,
+            int                const cols, 
+            int                const rows, 
+            int                const format,
+            unsigned int       const cBitCount, 
+            struct pixelformat const pixelformat,
+            xel                      colormap[]) {
+/*----------------------------------------------------------------------------
+  Write the PNM raster to Standard Output, corresponding to the raw BMP
+  raster BMPraster.  Write the raster assuming the PNM image has 
+  dimensions 'cols' by 'rows' and format 'format', with maxval 255.
+
+  The BMP image has 'cBitCount' bits per pixel, arranged in format
+  'pixelformat'.
+
+  If the image is colormapped, colormap[] is the colormap
+  (colormap[i] is the color with color index i).
+-----------------------------------------------------------------------------*/
+    xel * xelrow;
+    int row;
+
+    xelrow = pnm_allocrow(cols);
+
+    for (row = 0; row < rows; ++row) {
+        convertRow(BMPraster[row], xelrow, cols, cBitCount, pixelformat,
+                   colormap);
+        pnm_writepnmrow(stdout, xelrow, cols, bmpMaxval, format, FALSE);
+    }
+    pnm_freerow(xelrow);
+}
+
+
+
 int
 main(int argc, char ** argv) {
 
@@ -713,7 +1061,9 @@ main(int argc, char ** argv) {
            though the bottom row comes first in the BMP format.
         */
     unsigned int cBitCount;
-    enum bmpClass class;
+        /* Number of bits in BMP raster for each pixel */
+    struct pixelformat pixelformat;
+        /* Format of the raster bits for a single pixel */
     xel *colormap;
         /* Malloc'ed colormap (palette) from the BMP.  Undefined if
            not a colormapped BMP.
@@ -730,7 +1080,7 @@ main(int argc, char ** argv) {
         ifname = cmdline.input_filespec;
 
     readBmp(ifP, &BMPraster, &cols, &rows, &grayPresent, &colorPresent, 
-            &cBitCount, &class, &colormap,
+            &cBitCount, &pixelformat, &colormap,
             cmdline.verbose);
     pm_close(ifP);
 
@@ -746,7 +1096,7 @@ main(int argc, char ** argv) {
     }
     pnm_writepnminit(stdout, cols, rows, bmpMaxval, outputType, FALSE);
 
-    writeRaster(BMPraster, cols, rows, outputType, cBitCount, class,
+    writeRaster(BMPraster, cols, rows, outputType, cBitCount, pixelformat,
                 colormap);
 
     free(colormap);
@@ -754,6 +1104,3 @@ main(int argc, char ** argv) {
 
     exit(0);
 }
-
-
-
