@@ -30,6 +30,7 @@
 #include <jpeglib.h>
 #include "pnm.h"
 #include "shhopt.h"
+#include "mallocvar.h"
 
 #define EXIT_WARNING 2   /* Goes with EXIT_SUCCESS, EXIT_FAILURE in stdlib.h */
 
@@ -155,7 +156,7 @@ interpret_density(const char *        const densityString,
         int matched;
         int horiz, vert;
 
-        unitName = malloc(strlen(densityString));
+        unitName = malloc(strlen(densityString)+1);
     
         matched = sscanf(densityString, "%dx%d%s", &horiz, &vert, unitName);
 
@@ -225,8 +226,10 @@ parse_command_line(const int argc, char ** argv,
     unsigned int option_def_index;
 
     int argc_parse;       /* argc, except we modify it as we parse */
-    char ** const argv_parse = malloc(argc*sizeof(char *));
-    /* argv, except we modify it as we parse */
+    char ** argv_parse;
+        /* argv, except we modify it as we parse */
+
+    MALLOCARRAY(argv_parse, argc);
 
     option_def_index = 0;   /* incremented by OPTENTRY */
     OPTENT3(0, "verbose",     OPT_FLAG,   NULL, &cmdline_p->verbose,        0);
