@@ -60,6 +60,7 @@
 #define VERSION "2.37.6 (21 July 2001) +netpbm"
 #include "pnm.h"
 #include "pngtxt.h"
+#include "mallocvar.h"
 
 typedef struct cahitem {
     xel color;
@@ -209,7 +210,7 @@ alloccoloralphahash(void)  {
     coloralphahash_table caht;
     int i;
 
-    caht = (coloralphahash_table) malloc(HASH_SIZE * sizeof(cahitem *));
+    MALLOCARRAY(caht,HASH_SIZE);
     if (caht == NULL)
         pm_error( "out of memory allocating hash table" );
 
@@ -246,7 +247,7 @@ addtocoloralphahash(coloralphahash_table const caht,
     int hash;
     cahitem * itemP;
 
-    itemP = (cahitem *) malloc(sizeof(cahitem));
+    MALLOCVAR(itemP);
     if (itemP == NULL)
         pm_error("Out of memory building hash table");
     hash = hashpixelalpha(*colorP, *alphaP);
@@ -812,8 +813,7 @@ compute_alpha_palette(FILE *         const ifP,
     cht = ppm_colorhisttocolorhash (chv, colors);
 
     for (color_index = 0 ; color_index < colors + 1 ; ++color_index) {
-        alphas_of_color[color_index] = 
-            (gray *)malloc (MAXCOLORS * sizeof (int));
+        MALLOCARRAY(alphas_of_color[color_index], MAXCOLORS);
         if (alphas_of_color[color_index] == NULL)
             pm_error ("out of memory allocating alpha/palette entries");
         alphas_of_color_cnt[color_index] = 0;
@@ -1771,7 +1771,8 @@ convertpnm (FILE * const ifp,
   /* Write the raster */
  
   /* max: 3 color channels, one alpha channel, 16-bit */
-  if ((line = (png_byte *) malloc (cols*8)) == NULL)
+  MALLOCARRAY(line, cols * 8);
+  if (line == NULL)
   {
     png_destroy_write_struct (&png_ptr, &info_ptr);
     pm_closer (ifp);
