@@ -167,9 +167,9 @@ static void
 add_ps_to_filespec(const char orig_filespec[], char ** const new_filespec_p,
                    const int verbose) {
 /*----------------------------------------------------------------------------
-   If orig_filespec[] names an existing file, but the same name with ".ps"
-   added to the end does, return the name with the .ps attached.  Otherwise,
-   just return orig_filespec[].
+   If orig_filespec[] does not name an existing file, but the same
+   name with ".ps" added to the end does, return the name with the .ps
+   attached.  Otherwise, just return orig_filespec[].
 
    Return the name in newly malloc'ed storage, pointed to by
    *new_filespec_p.
@@ -182,17 +182,16 @@ add_ps_to_filespec(const char orig_filespec[], char ** const new_filespec_p,
     if (stat_rc == 0)
         *new_filespec_p = strdup(orig_filespec);
     else {
-        char *filespec_plus_ps;
+        const char *filespec_plus_ps;
 
-        filespec_plus_ps = malloc(strlen(orig_filespec) + 4);
-        strcpy(filespec_plus_ps, orig_filespec);
-        strcat(filespec_plus_ps, ".ps");
+        asprintfN(&filespec_plus_ps, "%s.ps", orig_filespec);
 
         stat_rc = lstat(filespec_plus_ps, &statbuf);
         if (stat_rc == 0)
             *new_filespec_p = strdup(filespec_plus_ps);
         else
             *new_filespec_p = strdup(orig_filespec);
+        strfree(filespec_plus_ps);
     }
     if (verbose)
         pm_message("Input file is %s", *new_filespec_p);
@@ -258,24 +257,36 @@ enum postscript_language {COMMON_POSTSCRIPT, ENCAPSULATED_POSTSCRIPT};
 
 static enum postscript_language
 language_declaration(const char input_filespec[], int const verbose) {
-
-    FILE *infile;
+/*----------------------------------------------------------------------------
+  Return the Postscript language in which the file declares it is written.
+  (Except that if the file is on Standard Input or doesn't validly declare
+  a languages, just say it is Common Postscript).
+-----------------------------------------------------------------------------*/
     enum postscript_language language;
-    char line[80];
 
-    infile = pm_openr(input_filespec);
-
-    if (fgets(line, sizeof(line), infile) == NULL)
+    if (strcmp(input_filespec, "-") == 0)
+        /* Can't read stdin, because we need it to remain positioned for the 
+           Ghostscript interpreter to read it.
+        */
         language = COMMON_POSTSCRIPT;
     else {
-        const char eps_header[] = "%!PS-Adobe EPSF-3.0";
+        FILE *infile;
+        char line[80];
 
-        if (strncmp(line, eps_header, sizeof(eps_header)) == 0)
-            language = ENCAPSULATED_POSTSCRIPT;
-        else
+        infile = pm_openr(input_filespec);
+
+        if (fgets(line, sizeof(line), infile) == NULL)
             language = COMMON_POSTSCRIPT;
+        else {
+            const char eps_header[] = "%!PS-Adobe EPSF-3.0";
+            
+            if (strncmp(line, eps_header, sizeof(eps_header)) == 0)
+                language = ENCAPSULATED_POSTSCRIPT;
+            else
+                language = COMMON_POSTSCRIPT;
+        }
+        fclose(infile);
     }
-    fclose(infile);
     if (verbose)
         pm_message("language is %s",
                    language == ENCAPSULATED_POSTSCRIPT ?
@@ -303,8 +314,8 @@ compute_box_to_extract(const struct box cmdline_extract_box,
         struct box ps_bb;  /* Box described by %%BoundingBox stmt in input */
 
         if (strcmp(input_filespec, "-") == 0)
-            /* Can't read stdin, because we need it positioned for the 
-               Ghostscript interpreter to read it.
+            /* Can't read stdin, because we need it to remain
+               positioned for the Ghostscript interpreter to read it.  
             */
             ps_bb.llx = -1;
         else {
@@ -434,72 +445,87 @@ add_borders(const struct box input_box,
 
 
 
-static char *
+static const char *
 compute_pstrans(const struct box box, const enum orientation orientation,
                 const int xsize, const int ysize, 
                 const int xres, const int yres) {
 
-    static char retval[100]; 
+    const char * retval;
 
     if (orientation == PORTRAIT) {
         int llx, lly;
         llx = box.llx - (xsize * 72 / xres - (box.urx - box.llx)) / 2;
         lly = box.lly - (ysize * 72 / yres - (box.ury - box.lly)) / 2;
-        sprintf(retval, "%d neg %d neg translate", llx, lly);
+        asprintfN(&retval, "%d neg %d neg translate", llx, lly);
     } else {
         int llx, ury;
         llx = box.llx - (ysize * 72 / yres - (box.urx - box.llx)) / 2;
         ury = box.ury + (xsize * 72 / xres - (box.ury - box.lly)) / 2;
-        sprintf(retval, "90 rotate %d neg %d neg translate", llx, ury);
+        asprintfN(&retval, "90 rotate %d neg %d neg translate", llx, ury);
     }
+
+    if (retval == NULL)
+        pm_error("Unable to allocate memory for pstrans");
+
     return retval;
 }
 
 
 
-static char *
+static const char *
 compute_outfile_arg(const struct cmdline_info cmdline) {
 
-    char *retval;  /* malloc'ed */
-
-    retval = malloc(strlen(cmdline.input_filespec) + 10);
+    const char *retval;  /* malloc'ed */
 
     if (cmdline.goto_stdout)
-        strcpy(retval, "-");
+        retval = strdup("-");
     else if (strcmp(cmdline.input_filespec, "-") == 0)
-        strcpy(retval, "-");
+        retval = strdup("-");
     else {
-        strcpy(retval, cmdline.input_filespec);
-        if (strlen(retval) > 3 && strcmp(retval+strlen(retval)-3, ".ps") == 0) 
+        char * basename;
+        const char * suffix;
+        
+        basename  = strdup(cmdline.input_filespec);
+        if (strlen(basename) > 3 && 
+            strcmp(basename+strlen(basename)-3, ".ps") == 0) 
             /* The input filespec ends in ".ps".  Chop it off. */
-            retval[strlen(retval)-3] = '\0';
+            basename[strlen(basename)-3] = '\0';
 
-        strcat(retval, "%03d.");
         switch (cmdline.format_type) {
-        case PBM_TYPE: strcat(retval, "pbm"); break;
-        case PGM_TYPE: strcat(retval, "pgm"); break;
-        case PPM_TYPE: strcat(retval, "ppm"); break;
-        default: pm_error("Internal error: invalid value format_type");
+        case PBM_TYPE: suffix = "pbm"; break;
+        case PGM_TYPE: suffix = "pgm"; break;
+        case PPM_TYPE: suffix = "ppm"; break;
+        default: pm_error("Internal error: invalid value for format_type: %d",
+                          cmdline.format_type);
         }
+        asprintfN(&retval, "%s%%03d.%s", basename, suffix);
+
+        strfree(basename);
     }
     return(retval);
 }
 
 
 
-static char *
+static const char *
 compute_gs_device(const int format_type, const int forceplain) {
 
-    static char retval[20];
+    const char * basetype;
+    const char * retval;
 
     switch (cmdline.format_type) {
-    case PBM_TYPE: strcpy(retval, "pbm"); break;
-    case PGM_TYPE: strcpy(retval, "pgm"); break;
-    case PPM_TYPE: strcpy(retval, "ppm"); break;
+    case PBM_TYPE: basetype = "pbm"; break;
+    case PGM_TYPE: basetype = "pgm"; break;
+    case PPM_TYPE: basetype = "ppm"; break;
     default: pm_error("Internal error: invalid value format_type");
     }
-    if (!forceplain)
-        strcat(retval, "raw");
+    if (forceplain)
+        retval = strdup(basetype);
+    else
+        asprintfN(&retval, "%sraw", basetype);
+
+    if (retval == NULL)
+        pm_error("Unable to allocate memory for gs device");
 
     return(retval);
 }
@@ -720,9 +746,9 @@ main(int argc, char **argv) {
 
     enum postscript_language language;
     enum orientation orientation;
-    char *ghostscript_device;
-    char *outfile_arg;
-    char *pstrans;
+    const char *ghostscript_device;
+    const char *outfile_arg;
+    const char *pstrans;
 
     pnm_init(&argc, argv);
 
@@ -758,6 +784,10 @@ main(int argc, char **argv) {
                         xsize, ysize, xres, yres, input_filespec,
                         language, cmdline.verbose);
 
+    strfree(ghostscript_device);
+    strfree(outfile_arg);
+    strfree(pstrans);
+    
     return 0;
 }
 
