@@ -27,6 +27,7 @@
 
 #include "ppm.h"
 #include "ppmdraw.h"
+#include "nstring.h"
 #ifdef DEBUG
 #include <assert.h>
 #else
@@ -439,8 +440,8 @@ static void slidefind(sname, dironly, ucasen)
 	    if (ch == EOS) {
 		break;
 	    }
-	    if (ucasen && islower(ch)) {
-		ch = toupper(ch);
+	    if (ucasen && ISLOWER(ch)) {
+		ch = TOUPPER(ch);
 	    }
 	    uname[i] = ch;
 	}
