@@ -14,6 +14,7 @@
 
 #include "pnm.h"
 #include "shhopt.h"
+#include "mallocvar.h"
 
 #ifndef M_PI
 #define M_PI    3.14159265358979323846
@@ -508,7 +509,7 @@ shearImageY(xel ** const inxels,
     struct shearParm * shearParm;  /* malloc'ed */
     int col;
 
-    shearParm = malloc(cols * sizeof(shearParm[0]));
+    MALLOCARRAY(shearParm, cols);
     if (shearParm == NULL)
         pm_error("Unable to allocate memory for shearParm");
 
