@@ -5,9 +5,16 @@
    that deal with the PAM (Portable Arbitrary Format) image format.
 -----------------------------------------------------------------------------*/
 
+/* See libpbm.c for the complicated explanation of this 32/64 bit file
+   offset stuff.
+*/
+#define _FILE_OFFSET_BITS 64
+#define _LARGE_FILES  
+
 #include <string.h>
 #include <limits.h>
 
+#include "mallocvar.h"
 #include "nstring.h"
 #include "pam.h"
 #include "ppm.h"
@@ -247,7 +254,7 @@ pnm_allocpamarray(const struct pam * const pamP) {
        by allocating one large chunk.
        */
     
-    tuplearray = malloc(pamP->height * sizeof(tuple*));
+    MALLOCARRAY(tuplearray, pamP->height);
     if (tuplearray == NULL) 
         pm_error("Out of memory allocating the row pointer section of "
                  "a %u row array", pamP->height);
@@ -768,7 +775,8 @@ pnm_writepamrow(const struct pam *const pamP, const tuple * const tuplerow) {
 
 
 void
-pnm_checkpam(const struct pam *const pamP, const enum pm_check_type check_type, 
+pnm_checkpam(const struct pam *   const pamP, 
+             enum pm_check_type   const check_type, 
              enum pm_check_code * const retval_p) {
 
     if (check_type != PM_CHECK_BASIC) {
