@@ -22,10 +22,13 @@
 
        Likewise for all the other C library file functions.
 
-       And off_t and fpos_t are 64 bit types instead of 32.
+       And off_t and fpos_t are 64 bit types instead of 32.  Consequently,
+       pm_filepos_t might be 64 bits instead of 32.
     */
 #define _LARGE_FILES  
     /* This does for AIX what _FILE_OFFSET_BITS=64 does for GNU */
+#define _LARGE_FILE_API
+    /* This makes the the x64() functions available on AIX */
 
 #include <stdio.h>
 #include "version.h"
@@ -40,17 +43,7 @@
   #include <io.h>
 #endif
 
-/* In GNU, _LFS_LARGEFILE means the "off_t" functions are available.
-   In AIX, _LARGE_FILE_API says that.
-*/
-#if defined(_LFS_LARGEFILE) || defined(_LARGE_FILE_API)
-    /* The C library has ftello() and fseeko() */
-#else
-#define ftello ftell
-#define fseeko fseek
-#endif
-
-
+#include "mallocvar.h"
 
 /* The following are set by pm_init(), then used by subsequent calls to other
    pm_xxx() functions.
@@ -120,10 +113,10 @@ pm_error(const char format[], ... ) {
 
 char*
 pm_allocrow(int const cols, int const size) {
-    register char* itrow;
+    char * itrow;
 
-    itrow = (char*) malloc( cols * size );
-    if ( itrow == (char*) 0 )
+    itrow = malloc( cols * size );
+    if ( itrow == NULL )
         pm_error( "out of memory allocating a row" );
     return itrow;
     }
@@ -134,57 +127,77 @@ pm_freerow(char * const itrow) {
 }
 
 
-#ifndef A_FRAGARRAY
+
 char**
 pm_allocarray(int const cols, int const rows, int const size )  {
-    char** its;
-    int i;
+/*----------------------------------------------------------------------------
+   Allocate an array of 'rows' rows of 'cols' columns each, with each
+   element 'size' bytes.
 
-    its = (char**) malloc( rows * sizeof(char*) );
-    if ( its == (char**) 0 )
-        pm_error( "out of memory allocating an array" );
-    its[0] = (char*) malloc( rows * cols * size );
-    if ( its[0] == (char*) 0 )
-        pm_error( "out of memory allocating an array" );
-    for ( i = 1; i < rows; ++i )
-        its[i] = &(its[0][i * cols * size]);
-    return its;
+   We use a special format where we tack on an extra element to the row
+   index to indicate the format of the array.
+
+   We have two ways of allocating the space: fragmented and
+   unfragmented.  In both, the row index (plus the extra element) is
+   in one block of memory.  In the fragemented format, each row is
+   also in an independent memory block, and the extra row pointer is
+   NULL.  In the unfragmented format, all the rows are in a single
+   block of memory called the row heap and the extra row pointer is
+   the address of that block.
+
+   We use unfragmented format if possible, but if the allocation of the
+   row heap fails, we fall back to fragmented.
+-----------------------------------------------------------------------------*/
+    char** rowIndex;
+    char * rowheap;
+
+    MALLOCARRAY(rowIndex, rows + 1);
+    if ( rowIndex == NULL )
+        pm_error("out of memory allocating row index (%u rows) for an array",
+                 rows);
+    rowheap = malloc( rows * cols * size );
+    if ( rowheap == NULL ) {
+        /* We couldn't get the whole heap in one block, so try fragmented
+           format.
+        */
+        unsigned int row;
+        
+        rowIndex[rows] = NULL;   /* Declare it fragmented format */
+
+        for (row = 0; row < rows; ++row) {
+            rowIndex[row] = pm_allocrow(cols, size);
+            if (rowIndex[row] == NULL)
+                pm_error("out of memory allocating Row %u "
+                         "(%u columns, %u bytes per tuple) "
+                         "of an array", row, cols, size);
+        }
+    } else {
+        /* It's unfragmented format */
+        unsigned int row;
+        rowIndex[rows] = rowheap;  /* Declare it unfragmented format */
+
+        for (row = 0; row < rows; ++row)
+            rowIndex[row] = &(rowheap[row * cols * size]);
     }
-
-void
-pm_freearray(char ** const its, int const rows) {
-    free( its[0] );
-    free( its );
-}
-#else /* A_FRAGARRAY */
-char**
-pm_allocarray(int const cols, int const rows, int const size) {
-    char** its;
-    int i;
-    its = (char**) malloc( (rows + 1) * sizeof(char*) );
-    if ( its == (char**) 0 )
-        pm_error( "out of memory allocating an array" );
-    its[rows] = its[0] = (char*) malloc( rows * cols * size );
-    if ( its[0] != (char*) 0 )
-        for ( i = 1; i < rows; ++i )
-            its[i] = &(its[0][i * cols * size]);
-    else
-        for( i = 0; i < rows; ++i )
-            its[i] = pm_allocrow( cols, size );
-    return its;
+    return rowIndex;
 }
 
 void
-pm_freearray(char ** const its, int const rows) {
-    int i;
-    if( its[rows] != (char*) 0 )
-        free( its[rows] );
-    else
-        for( i = 0; i < rows; ++i )
-            pm_freerow( its[i] );
-    free( its );
+pm_freearray(char ** const rowIndex, 
+             int     const rows) {
+
+    void * const rowheap = rowIndex[rows];
+
+    if (rowheap != NULL)
+        free(rowheap);
+    else {
+        unsigned int row;
+        for (row = 0; row < rows; ++row)
+            pm_freerow(rowIndex[row]);
+    }
+    free(rowIndex);
 }
-#endif /* A_FRAGARRAY */
+
 
 
 /* Case-insensitive keyword matcher. */
@@ -1048,6 +1061,10 @@ pm_check(FILE *               const file,
     pm_filepos curpos;  /* Current position of file; -1 if none */
     int rc;
 
+#ifdef LARGEFILEDEBUG
+    pm_message("pm_filepos received by pm_check() is %u bytes",
+               sizeof(pm_filepos));
+#endif
     /* Note: ftello may actually by ftello64 or ftell, depending on the 
        capabilities of the underlying C library.
     */
