@@ -14,6 +14,7 @@
 
 #include "ppm.h"
 #include "libppm.h"
+#include "mallocvar.h"
 
 #define HASH_SIZE 20023
 
@@ -176,8 +177,7 @@ computecolorhash(pixel ** const pixels,
                     ppm_freecolorhash(cht);
                     return NULL;
                 } else {
-                    chl = (colorhist_list) 
-                        malloc(sizeof(struct colorhist_list_item));
+                    MALLOCVAR(chl);
                     if (chl == NULL)
                         pm_error("out of memory computing hash table");
                     chl->ch.color = apixel;
@@ -222,8 +222,8 @@ ppm_alloccolorhash(void)  {
     colorhash_table cht;
     int i;
 
-    cht = (colorhash_table) malloc(HASH_SIZE * sizeof(colorhist_list));
-    if (cht == 0)
+    MALLOCARRAY(cht, HASH_SIZE);
+    if (cht == NULL)
         pm_error( "out of memory allocating hash table" );
 
     for (i = 0; i < HASH_SIZE; ++i)
@@ -235,20 +235,26 @@ ppm_alloccolorhash(void)  {
 
 
 int
-ppm_addtocolorhash( colorhash_table cht, const pixel * const colorP, 
-                    const int value ) {
-    register int hash;
-    register colorhist_list chl;
+ppm_addtocolorhash(colorhash_table const cht, 
+                   const pixel *   const colorP, 
+                   int             const value) {
 
-    chl = (colorhist_list) malloc( sizeof(struct colorhist_list_item) );
+    int retval;
+    colorhist_list chl;
+
+    MALLOCVAR(chl);
     if (chl == NULL)
-        return -1;
-    hash = ppm_hashpixel(*colorP);
-    chl->ch.color = *colorP;
-    chl->ch.value = value;
-    chl->next = cht[hash];
-    cht[hash] = chl;
-    return 0;
+        retval = -1;
+    else {
+        int const hash = ppm_hashpixel(*colorP);
+
+        chl->ch.color = *colorP;
+        chl->ch.value = value;
+        chl->next = cht[hash];
+        cht[hash] = chl;
+        retval = 0;
+    }
+    return retval;
 }
 
 
@@ -293,8 +299,7 @@ ppm_colorhashtocolorhist(colorhash_table const cht, int const maxcolors) {
         chvSize = maxcolors;
 
     /* Collate the hash table into a simple colorhist array. */
-    chv = (colorhist_vector) malloc(chvSize * sizeof(struct colorhist_item));
-    /* (Leave room for expansion by caller.) */
+    MALLOCARRAY(chv, chvSize);
     if (chv == NULL)
         pm_error("out of memory generating histogram");
 
@@ -315,8 +320,8 @@ ppm_colorhashtocolorhist(colorhash_table const cht, int const maxcolors) {
 
 
 colorhash_table
-ppm_colorhisttocolorhash( const colorhist_vector chv, const int colors )
-    {
+ppm_colorhisttocolorhash(colorhist_vector const chv, 
+                         int              const colors) {
     colorhash_table cht;
     int i, hash;
     pixel color;
@@ -324,26 +329,26 @@ ppm_colorhisttocolorhash( const colorhist_vector chv, const int colors )
 
     cht = ppm_alloccolorhash( );  /* Initializes to NULLs */
 
-    for ( i = 0; i < colors; ++i )
-    {
-    color = chv[i].color;
-    hash = ppm_hashpixel( color );
-    for ( chl = cht[hash]; chl != (colorhist_list) 0; chl = chl->next )
-        if ( PPM_EQUAL( chl->ch.color, color ) )
-        pm_error(
-            "same color found twice - %d %d %d", PPM_GETR(color),
-            PPM_GETG(color), PPM_GETB(color) );
-    chl = (colorhist_list) malloc( sizeof(struct colorhist_list_item) );
-    if ( chl == (colorhist_list) 0 )
-        pm_error( "out of memory" );
-    chl->ch.color = color;
-    chl->ch.value = i;
-    chl->next = cht[hash];
-    cht[hash] = chl;
+    for (i = 0; i < colors; ++i) {
+        color = chv[i].color;
+        hash = ppm_hashpixel(color);
+        for (chl = cht[hash]; chl != (colorhist_list) 0; chl = chl->next)
+            if (PPM_EQUAL(chl->ch.color, color))
+                pm_error(
+                    "same color found twice - %d %d %d", PPM_GETR(color),
+                    PPM_GETG(color), PPM_GETB(color) );
+        MALLOCVAR(chl);
+        if (chl == NULL)
+            pm_error("out of memory");
+        chl->ch.color = color;
+        chl->ch.value = i;
+        chl->next = cht[hash];
+        cht[hash] = chl;
     }
-
     return cht;
-    }
+}
+
+
 
 int
 ppm_lookupcolor( const colorhash_table cht, const pixel * const colorP )
@@ -566,35 +571,33 @@ ppm_addtocolorrow(colorrow, ncolorsP, maxcolors, pixelP)
 
 
 int
-ppm_findclosestcolor(colormap, ncolors, pP)
-    pixel *colormap;
-    int ncolors;
-    pixel *pP;
-{
+ppm_findclosestcolor(const pixel * const colormap, 
+                     int           const ncolors, 
+                     const pixel * const pP) {
+    
     /* Search colormap for closest match.       */
     /* algorithm taken from ppmquant.c   -IUW   */
-    register int i, r1, g1, b1;
+    int const r1 = PPM_GETR(*pP);
+    int const g1 = PPM_GETG(*pP);
+    int const b1 = PPM_GETB(*pP);
+
+    int i;
     int ind;
-    long dist;
+    unsigned int bestDist;
 
-    r1 = PPM_GETR(*pP);
-    g1 = PPM_GETG(*pP);
-    b1 = PPM_GETB(*pP);
-    dist = 2000000000;
-    for( i = 0; i < ncolors; i++ ) {
-        register int r2, g2, b2;
-        long newdist;
+    bestDist = UINT_MAX;
+    ind = -1;
 
-        r2 = PPM_GETR(colormap[i]);
-        g2 = PPM_GETG(colormap[i]);
-        b2 = PPM_GETB(colormap[i]);
-        newdist = ( r1 - r2 ) * ( r1 - r2 ) +
-                  ( g1 - g2 ) * ( g1 - g2 ) +
-                  ( b1 - b2 ) * ( b1 - b2 );
+    for(i = 0; i < ncolors && bestDist > 0; ++i) {
+        int const r2 = PPM_GETR(colormap[i]);
+        int const g2 = PPM_GETG(colormap[i]);
+        int const b2 = PPM_GETB(colormap[i]);
 
-        if( newdist < dist ) {
+        unsigned int const dist = SQR(r1 - r2) + SQR(g1 - g2) + SQR(b1 - b2);
+        
+        if (dist < bestDist ) {
             ind = i;
-            dist = newdist;
+            bestDist = dist;
         }
     }
     return ind;
